--[[--------------------------------------------------------------
Disenchant Crafted Items
by Sunsequent

A button in profession window for fast disenchanting of crafted 
items without the need to target them in bags.

Version 2.1.3_WotLK
Release Date 2022.09.13
--------------------------------------------------------------]]--

---- Version 2.1.3_WotLK
-- Updated for Wrath of the Lich King Classic

---- Version 2.1.1_TBC
-- Updated for Burning Crusade Classic

---- Version 2.0.1_Classic
-- Fixed a crash on collapsing a section in tradeskill window when having an item selected in that section

---- Version 2.0.0_Classic
-- Updated for Classic

-- Configuration Parameters
local ignoreSoulbound = true;
local autoLoot = true;

-- Variables
local tradeSkillWindowLoaded = false;
local selectedItemID;
local numberOfItems = 0;
local itemLocations = {};
local disenchantCasting = false;
local itemLocked = false;
local lootOpen = false;
local locationDisenchantStarted;
local previousItemBagSlot;
local autoLootCurrent = 0;

-- Localization
local locale;
local disenchantName;
local itemCountText;
local buttonWidth;
local buttonLocX;

SLASH_DISENCHANTCRAFTEDITEMS1, SLASH_DISENCHANTCRAFTEDITEMS2 = "/disenchantcrafteditems", "/dci";

BINDING_HEADER_DISENCHANTCRAFTEDITEMS = "Disenchant Crafted Items";

function SlashCmdList.DISENCHANTCRAFTEDITEMS(msg, editbox)
    if msg == "autoloot" then
        if autoLoot == true then
			autoLoot = false;
			DCI_AutoLootDisenchanted = autoLoot;
			
			print("Disenchanted materials are not autolooted.");
		else
			autoLoot = true;
			DCI_AutoLootDisenchanted = autoLoot;
			
			print("Disenchanted materials are autolooted.");
		end

        return;
    end
	
	if msg == "soulbound" then
        if ignoreSoulbound == true then
			ignoreSoulbound = false;
			DCI_IgnoreSoulbound = ignoreSoulbound;
			
			print("Soulbound crafted items are included in disenchanting.");
		else
			ignoreSoulbound = true;
			DCI_IgnoreSoulbound = ignoreSoulbound;
			
			print("Soulbound crafted items are excluded from disenchanting.");
		end

        return;
    end
	
	if msg == "status" then
		if ignoreSoulbound == false then
			print("Soulbound crafted items are included in disenchanting.");
		else
			print("Soulbound crafted items are excluded from disenchanting.");
		end
		
		if autoLoot == false then
			print("Disenchanted materials are not autolooted.");
		else
			print("Disenchanted materials are autolooted.");
		end
		
		return;
	end
	
	print("Disenchant Crafted Items (also /dci) commands:");
	print("/disenchantcrafteditems autoloot");
	print("/disenchantcrafteditems soulbound");
	print("/disenchantcrafteditems status");
end

local function SetLocalization()
	-- Localization:
	-- enUS Items in backpack			Disenchant
	-- deDE Gegenstände im Rucksack		Entzaubern
	-- esES Artículos en la mochila		Desencantar
	-- esMX Artículos en la mochila		Desencantar
	-- frFR Articles dans le sac à dos	Désenchanter
	-- itIT Oggetti nello zaino			Disincantamento
	-- koKR 배낭에있는 물건들			마력 추출
	-- ptBR Itens na mochila			Desencantar
	-- ruRU Предметы в рюкзаке			Распыление
	-- zhCN 背包里的物品					分解
	-- zhTW 背包里的物品					分解
	
	locale = GetLocale();
	
	disenchantName = GetSpellInfo(13262);
	_G["BINDING_NAME_CLICK DisenchantCraftedItemsButton:LeftButton"] = disenchantName.." Button"
	
	buttonWidth = 100;
	buttonLocX = -23;
	
	if locale == "enUS" then
		itemCountText = "Items in backpack";
	elseif locale == "deDE" then
		itemCountText = "Gegenstände im Rucksack";
	elseif locale == "esES" then
		itemCountText = "Artículos en la mochila";
	elseif locale == "esMX" then
		itemCountText = "Artículos en la mochila";
	elseif locale == "frFR" then
		itemCountText = "Articles dans le sac à dos";
	elseif locale == "itIT" then
		itemCountText = "Oggetti nello zaino";
		buttonWidth = 120;
		buttonLocX = -43;
	elseif locale == "koKR" then
		itemCountText = "배낭에있는 물건들";
	elseif locale == "ptBR" then
		itemCountText = "Itens na mochila";
	elseif locale == "ruRU" then
		itemCountText = "Предметы в рюкзаке";
	elseif locale == "zhCN" then
		itemCountText = "背包里的物品";
	elseif locale == "zhTW" then
		itemCountText = "背包里的物品";
	end
end

local function TableHasValue(tab, val)
	if val ~= nil then
		for index, value in ipairs(tab) do
			if value == val then
				return true
			end
		end
	end
	
    return false
end

local function TableContainsValue(tab, val)
	if val ~= nil then
		for index, value in ipairs(tab) do
			if string.find(val, value) then
				return true
			end
		end
	end
	
    return false
end

local function StringSplit(inputString, separator)
	if separator == nil then
		separator = "%s";
	end
	
	local splitTable = {}; 
	local splitCounter = 1;
	
	for str in string.gmatch(inputString, "([^"..separator.."]+)") do
		splitTable[splitCounter] = str;
		splitCounter = splitCounter + 1;
	end
	
	return splitTable;
end

local soulboundTooltip = CreateFrame("GameTooltip", "CheckSoulboundTooltip", nil, "GameTooltipTemplate")
local function IsSoulbound(bag, slot)
	soulboundTooltip:SetOwner(UIParent, "ANCHOR_NONE");
	soulboundTooltip:SetBagItem(bag, slot);
	soulboundTooltip:Show();
	
	for i = 1,soulboundTooltip:NumLines() do
		if(_G["CheckSoulboundTooltipTextLeft"..i]:GetText()==ITEM_SOULBOUND) then
			soulboundTooltip:Hide();
			return true;
		end
	end
	
	soulboundTooltip:Hide()
	return false
end

local function IsDisenchantable(itemID)
	if IsSpellKnown(13262) then
		if itemID ~= nil then
			local _, _, quality, _, _, _, _, _, _, _, _, class, subClass = GetItemInfo(tonumber(itemID))
			
			if(class == 2 or class == 4 or (class == 3 and subClass == 11)) then
				if quality > 1 and quality < 5 then
					return true;
				end
			end
		end
	end
	
	return false;
end

local function GetBagItemList(selectedItemID)
	itemLocations = {};
	local itemCounter = 0;
	
	for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots(bag) do
			itemID = GetContainerItemID(bag, slot)

            if(tonumber(itemID) == tonumber(selectedItemID)) then
				if (IsSoulbound(bag, slot) == false) then
					if IsDisenchantable(selectedItemID) == true then
						itemCounter = itemCounter + 1;
						itemLocations[itemCounter] = bag.." "..slot;
					end
				else
					if ignoreSoulbound == false then
						if IsDisenchantable(selectedItemID) == true then
							itemCounter = itemCounter + 1;
							itemLocations[itemCounter] = bag.." "..slot;
						end
					end					
				end
            end
        end
    end
	
	return itemCounter;
end

local button = CreateFrame("Button", "DisenchantCraftedItemsButton", UIParent, "SecureActionButtonTemplate");
local frameItemCount = CreateFrame("Frame", "DisenchantCraftedItemsCount")

local function ResetFrames()
	locationDisenchantStarted = nil;
	disenchantCasting = false;
	lootOpen = false;
	itemLocked = false;
	
	frameItemCount:Hide();
	button:Hide();
	button:SetAttribute("type1", "macro");
	button:SetAttribute("macrotext1", "");
end

local tailoringName;
local leatherworkingName;
local blacksmithingName;
local engineeringName;

local function GetProfessionNames()
	tailoringName = GetSpellInfo(3908);
	leatherworkingName = GetSpellInfo(2108);
	blacksmithingName = GetSpellInfo(2018);
	engineeringName = GetSpellInfo(4036);
	
	professionsForButton = { tailoringName, leatherworkingName, blacksmithingName, engineeringName };
end

local previousRecipeID = nil;

local function ShowButton()
	local recipeID = GetTradeSkillSelectionIndex();
	
	if recipeID ~= nil then
		if recipeID ~= previousRecipeID then
			previousRecipeID = recipeID;
			local itemLink = GetTradeSkillItemLink(recipeID);
			
			if itemLink ~= nil then
				selectedItemID = select(3, strfind(itemLink, "item:(%d+)"))

				numberOfItems = GetBagItemList(selectedItemID);
				frameItemCount.text:SetText(itemCountText..": "..numberOfItems);
				
				if previousItemBagSlot ~= itemLocations[1] then
					previousItemBagSlot = itemLocations[1];
					locationDisenchantStarted = nil;
					disenchantCasting = false;
					lootOpen = false;
					itemLocked = false;
				end
				
				if selectedItemID ~= nil then
					if numberOfItems > 0 and numberOfItems ~= nil then

						frameItemCount:Show();
						button:Show();
						
						local spell = nil;
						local bagID = nil;
						local slotID = nil;
						
						if (disenchantCasting == false and lootOpen == false and itemLocked == false and (locationDisenchantStarted == nil or locationDisenchantStarted ~= itemLocations[1])) then
							if (itemLocations[1] ~= nil) then
								spell = disenchantName;
								bagID = StringSplit(itemLocations[1], nil)[1];
								slotID = StringSplit(itemLocations[1], nil)[2];
							else
								spell = nil;
								bagID = nil;
								slotID = nil;
							end
						end
						
						if (disenchantCasting == false and lootOpen == false and itemLocked == false and locationDisenchantStarted == itemLocations[1]) then
							if (itemLocations[2] ~= nil) then
								spell = disenchantName;
								bagID = StringSplit(itemLocations[2], nil)[1];
								slotID = StringSplit(itemLocations[2], nil)[2];
							else
								spell = nil;
								bagID = nil;
								slotID = nil;
							end
						end
						
						if (itemLocations[1] ~= nil) then
							button:SetAttribute("type1", "spell");
							button:SetAttribute("spell1", spell);
							button:SetAttribute("target-bag1", bagID)
							button:SetAttribute("target-slot1", slotID)
						end
					else
						ResetFrames();
					end
				end
			else
				ResetFrames();
			end
		end
	else
		ResetFrames();
	end
end

local refreshInitialized = false;

local function Delay()
	if TradeSkillFrame:IsShown() == true then
		ShowButton();
		C_Timer.After(0.1, Delay);
	else
		refreshInitialized = false;
	end
end

local function RefreshSelectedTradeSkill()
	if refreshInitialized == false then 
		refreshInitialized = true;

		C_Timer.After(0.1, Delay);
	end
end

frameItemCount:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" then 
		loadedAddon = select(1, ...);
		
		if loadedAddon == "DisenchantCraftedItems" then
			if DCI_AutoLootDisenchanted == nil then
				DCI_AutoLootDisenchanted = autoLoot;
			else
				autoLoot = DCI_AutoLootDisenchanted;
			end
			
			if DCI_IgnoreSoulbound == nil then
				DCI_IgnoreSoulbound = ignoreSoulbound;
			else
				ignoreSoulbound = DCI_IgnoreSoulbound;
			end
		end
		
		if loadedAddon == "Blizzard_TradeSkillUI" then
			if (IsSpellKnown(13262)) then
				tradeSkillWindowLoaded = true;
				
				GetProfessionNames();
				SetLocalization();
				
				frameItemCount:SetParent(TradeSkillCancelButton)
				frameItemCount:SetSize(250,25);
				frameItemCount:SetPoint("TOPLEFT",TradeSkillCancelButton, "TOPLEFT", -242, 16)
				frameItemCount.text = frameItemCount.text or frameItemCount:CreateFontString(nil, "OVERLAY", "GameFontNormal") --GameFontNormal GameFontNormalLarge GameFontHighlight
				frameItemCount.text:SetAllPoints(true)
				frameItemCount.text:SetJustifyH("LEFT")
				frameItemCount.text:SetJustifyV("TOP")
				frameItemCount.text:SetTextColor(1,0.8046,0,1) --1,0.8046,0,1  1,0.84,0,1
			
			
				button:SetParent(TradeSkillCancelButton)
				button:SetSize(buttonWidth, 22) --Width, Height
				button:SetText(disenchantName)
				button:SetPoint("TOPLEFT", TradeSkillCancelButton, "TOPLEFT", buttonLocX, 23)
				
				button:SetNormalFontObject("GameFontNormal")
				
				local ntex = button:CreateTexture()
				ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
				ntex:SetTexCoord(0, 0.625, 0, 0.6875)
				ntex:SetAllPoints()	
				button:SetNormalTexture(ntex)

				local htex = button:CreateTexture()
				htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
				htex:SetTexCoord(0, 0.625, 0, 0.6875)
				htex:SetAllPoints()
				button:SetHighlightTexture(htex)

				local ptex = button:CreateTexture()
				ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
				ptex:SetTexCoord(0, 0.625, 0, 0.6875)
				ptex:SetAllPoints()
				button:SetPushedTexture(ptex)
			end
		end
	end
	
	if (IsSpellKnown(13262)) then
		if tradeSkillWindowLoaded == true then
			if TradeSkillFrame:IsShown() == true then
				local openedProfession = TradeSkillFrameTitleText:GetText();
				
				if TableContainsValue(professionsForButton, openedProfession) then
					RefreshSelectedTradeSkill();
					local recipeID = GetTradeSkillSelectionIndex();

					if recipeID ~= nil then
						local itemLink = GetTradeSkillItemLink(recipeID);
						
						if itemLink ~= nil then
							selectedItemID = select(3, strfind(itemLink, "item:(%d+)"))

							numberOfItems = GetBagItemList(selectedItemID);
							frameItemCount.text:SetText(itemCountText..": "..numberOfItems);
							
							if previousItemBagSlot ~= itemLocations[1] then
								previousItemBagSlot = itemLocations[1];
								locationDisenchantStarted = nil;
								disenchantCasting = false;
								lootOpen = false;
								itemLocked = false;
							end
							
							if selectedItemID ~= nil then
								if numberOfItems > 0 and numberOfItems ~= nil then
									frameItemCount:Show();
									button:Show();
									
									if event=="UNIT_SPELLCAST_START" then
										local unit, spellName, spellID = select(1,...)
										
										if (unit == "player" and spellID == 13262) then
											disenchantCasting = true;
										end
									end
									
									if event=="UNIT_SPELLCAST_SUCCEEDED" then
										local unit, spellName, spellID = select(1,...)
										
										if (unit == "player" and spellID == 13262) then
											locationDisenchantStarted = itemLocations[1];
											
											if autoLoot == true then
												if GetCVar("autoLootDefault") == "1" then 
													autoLootCurrent = 0;
												else
													autoLootCurrent = 1;
													SetCVar("autoLootDefault", 1);
												end
											end
										end
									end
									
									if event=="UNIT_SPELLCAST_STOP" then
										local unit, spellName, spellID = select(1,...)
										
										if (unit == "player" and spellID == 13262) then
											disenchantCasting = false;									
										end
									end
									
									if(event=="UNIT_SPELLCAST_INTERRUPTED") then
										local unit, spellName, spellID = select(1,...)
										
										if (unit == "player" and spellID == 13262) then
											disenchantCasting = false;
										end
									end
									
									if(event=="ITEM_LOCKED") then
										local bag, slot = select(1,...)
										
										if (bag ~= nil and slot ~= nil) then
											if (itemLocations[1] == bag.." "..slot) then
												itemLocked = true;
											end
										end
									end
									
									if(event=="ITEM_UNLOCKED") then
										local bag, slot = select(1,...)
										
										if (bag ~= nil and slot ~= nil) then
											if (itemLocations[1] == bag.." "..slot) then
												itemLocked = false;
											end
										else
											itemLocked = false;
										end
									end
									
									if(event=="LOOT_OPENED") then
										lootOpen = true;
									end
									
									if(event=="LOOT_CLOSED") then
										lootOpen = false;
										
										if autoLoot == true then
											if autoLootCurrent == 1 then
												SetCVar("autoLootDefault", 0);
											end
										end
									end

									local spell = nil;
									local bagID = nil;
									local slotID = nil;
									
									if (disenchantCasting == false and lootOpen == false and itemLocked == false and (locationDisenchantStarted == nil or locationDisenchantStarted ~= itemLocations[1])) then
										if (itemLocations[1] ~= nil) then
											spell = disenchantName;
											bagID = StringSplit(itemLocations[1], nil)[1];
											slotID = StringSplit(itemLocations[1], nil)[2];
										else
											spell = nil;
											bagID = nil;
											slotID = nil;
										end
									end
									
									if (disenchantCasting == false and lootOpen == false and itemLocked == false and locationDisenchantStarted == itemLocations[1]) then
										if (itemLocations[2] ~= nil) then
											spell = disenchantName;
											bagID = StringSplit(itemLocations[2], nil)[1];
											slotID = StringSplit(itemLocations[2], nil)[2];
										else
											spell = nil;
											bagID = nil;
											slotID = nil;
										end
									end
									
									if (itemLocations[1] ~= nil) then
										button:SetAttribute("type1", "spell");
										button:SetAttribute("spell1", spell);
										button:SetAttribute("target-bag1", bagID)
										button:SetAttribute("target-slot1", slotID)
									end
								else
									ResetFrames();
								end
							end
						else
							ResetFrames();
						end
					else
						ResetFrames();
					end
				else
					ResetFrames();
				end
			end
		end
	end
end)

frameItemCount:RegisterEvent("ADDON_LOADED");
frameItemCount:RegisterEvent("BAG_UPDATE");
frameItemCount:RegisterEvent("TRADE_SKILL_SHOW");
frameItemCount:RegisterEvent("TRADE_SKILL_UPDATE");
frameItemCount:RegisterEvent("TRADE_SKILL_LIST_UPDATE");
frameItemCount:RegisterEvent("UPDATE_TRADESKILL_RECAST");
frameItemCount:RegisterEvent("ITEM_LOCKED");
frameItemCount:RegisterEvent("ITEM_UNLOCKED");
frameItemCount:RegisterEvent("UNIT_SPELLCAST_START");
frameItemCount:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
frameItemCount:RegisterEvent("UNIT_SPELLCAST_STOP");
frameItemCount:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
frameItemCount:RegisterEvent("LOOT_OPENED");
frameItemCount:RegisterEvent("LOOT_CLOSED");
